package cn.ibizlab.assetmanagement.core.location.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.location.domain.EAMLocationStateModelLine;
import cn.ibizlab.assetmanagement.core.location.filter.EAMLocationStateModelLineSearchContext;
import cn.ibizlab.assetmanagement.core.location.service.IEAMLocationStateModelLineService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.location.mapper.EAMLocationStateModelLineMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[功能位置状态模型明细] 服务对象接口实现
 */
@Slf4j
@Service("EAMLocationStateModelLineServiceImpl")
public class EAMLocationStateModelLineServiceImpl extends ServiceImpl<EAMLocationStateModelLineMapper, EAMLocationStateModelLine> implements IEAMLocationStateModelLineService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationStateModelService eamlocationstatemodelService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationStateService eamlocationstateService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMLocationStateModelLine et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamlocationstatemodellineid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMLocationStateModelLine> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMLocationStateModelLine et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamlocationstatemodellineid",et.getEamlocationstatemodellineid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamlocationstatemodellineid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMLocationStateModelLine> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMLocationStateModelLine get(String key) {
        EAMLocationStateModelLine et = getById(key);
        if(et==null){
            et=new EAMLocationStateModelLine();
            et.setEamlocationstatemodellineid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMLocationStateModelLine getDraft(EAMLocationStateModelLine et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMLocationStateModelLine et) {
        return (!ObjectUtils.isEmpty(et.getEamlocationstatemodellineid()))&&(!Objects.isNull(this.getById(et.getEamlocationstatemodellineid())));
    }
    @Override
    @Transactional
    public boolean save(EAMLocationStateModelLine et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EAMLocationStateModelLine et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMLocationStateModelLine> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMLocationStateModelLine> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMLocationStateModelLine> selectByEamlocationstatemodelid(String eamlocationstatemodelid) {
        return baseMapper.selectByEamlocationstatemodelid(eamlocationstatemodelid);
    }

    @Override
    public void removeByEamlocationstatemodelid(String eamlocationstatemodelid) {
        this.remove(new QueryWrapper<EAMLocationStateModelLine>().eq("eamlocationstatemodelid",eamlocationstatemodelid));
    }

	@Override
    public List<EAMLocationStateModelLine> selectByEamlocationstateid(String eamlocationstateid) {
        return baseMapper.selectByEamlocationstateid(eamlocationstateid);
    }

    @Override
    public void removeByEamlocationstateid(String eamlocationstateid) {
        this.remove(new QueryWrapper<EAMLocationStateModelLine>().eq("eamlocationstateid",eamlocationstateid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMLocationStateModelLine> searchDefault(EAMLocationStateModelLineSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMLocationStateModelLine> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMLocationStateModelLine>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMLocationStateModelLine> getEamlocationstatemodellineByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMLocationStateModelLine> getEamlocationstatemodellineByEntities(List<EAMLocationStateModelLine> entities) {
        List ids =new ArrayList();
        for(EAMLocationStateModelLine entity : entities){
            Serializable id=entity.getEamlocationstatemodellineid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



